SYD(7)

# NAME

Overview of sandboxing with syd

# SANDBOXING

There are eight sandboxing types:

. *Read sandboxing*
. *Write sandboxing*
. *Exec sandboxing*
. *Stat sandboxing*
. *Network sandboxing*
. *Lock Sandboxing*
. *Memory Sandboxing*
. *PID sandboxing*

Sandboxing may be *on* or *off*:

|[ *off*
:< Sandboxing is off, none of the relevant system calls are checked and all access is allowed.
|[ *on*
:< Sandboxing defaults to deny, allowlists and denylists can be used to refine access rights.

In addition, there are filters for every sandboxing to prevent syd
from reporting an access violation. Note, access is still denied in such
cases.

## Read Sandboxing

This sandboxing checks certain system calls for filesystem read access. If a
system call tries to read a path, this attempt is reported and the system call
is denied. See the section called *Write Sandboxing* for more information on how
to customize this behaviour.

List of filtered system calls are: _open_(2), _openat_(2), _openat2_(2) and
_getrandom_(2).

- *getrandom* with the flag *GRND_RANDOM* is treated equivalent to \
*open(/dev/random, O_RDONLY)*.
- *getrandom* without the flag *GRND_RANDOM* is treated equivalent to \
*open(/dev/urandom, O_RDONLY)*.

Note, since 3.12.0, _open_(2) calls with the *O_PATH* flag are checked as part of
*Stat Sandboxing*.

## Write Sandboxing

This sandboxing checks certain system calls for filesystem write access. If a system
call tries to write, modify or change attributes of a path, this attempt is reported
in system log and the system call is denied. There are two ways to customize this
behaviour. syd may be configured to *allow* some path patterns. If the path
argument of the system call which is subject to be modified matches a pattern in the
list of allowed path patterns, this attempt is not denied. If, however it
matches a pattern in the list of *deny* path patterns the attempt is denied
(_last matching pattern wins_). Additionally, syd may be configured to
*filter* some path patterns. In this case a match will prevent syd from
reporting a warning about the access violation, the system call is still denied
though.

List of filtered system calls are: _chmod_(2), _fchmod_(2), _fchmodat_(2),
_chown_(2), _chown32_(2), _fchown_(2), _lchown_(2), _lchown32_(2),
_fchownat_(2), _open_(2), _openat_(2), _openat2_(2), _creat_(2), _mkdir_(2),
_mkdirat_(2), _mknod_(2), _mknodat_(2), _rmdir_(2), _truncate_(2),
_truncate64_(2), _mount_(2), _umount_(2), _umount2_(2), _utime_(2), _utimes_(2),
_utimensat_(2), _futimesat_(2), _unlink_(2), _unlinkat_(2), _link_(2),
_linkat_(2), _rename_(2), _renameat_(2), _renameat2_(2), _symlink_(2),
_symlinkat_(2), _setxattr_(2), _fsetxattr_(2), _lsetxattr_(2), _removexattr_(2),
_fremovexattr_(2) and _lremovexattr_(2).

## Exec Sandboxing

This sandboxing denies _execve_(2), and _execveat_(2) calls in case the path
argument does not match one of the allowed patterns. Note, all _exec_(2) family
functions are sandboxed because these functions are just wrappers of either one
of _execve_(2) or _execveat_(2) system calls. See the section called *Write
Sandboxing* for more information on how to customize this behaviour.

## Stat Sandboxing

This sandboxing checks certain system calls for filesystem statistics access.
This can be one of listing a directory, changing into a directory, or using a
_stat_(2) system call to query file metadata. This sandboxing type may be used to
effectively _hide files and directories_ from the sandbox process.

List of filtered system calls are: _access_(2), _faccessat_(2), _faccessat2_(2),
_chdir_(2), _fchdir_(2), _getdents_(2), _getdents64_(2), _stat_(2), _fstat_(2),
_lstat_(2), _statx_(2), _newfstatat_(2), _getxattr_(2), _lgetxattr_(2),
_fgetxattr_(2), _listxattr_(2), _flistxattr_(2), and _llistxattr_(2).

Note, since 3.12.0, _open_(2) calls with the *O_PATH* flag are checked as part of
*Stat Sandboxing*.

## Network Sandboxing

This sandboxing exposes a way to prevent unwanted network calls. The filtered
system calls are: _bind_(2), _connect_(2), _sendto_(2), _recvmsg_(2), and
_sendmsg_(2). To increase usability, these system calls are filtered in two
groups: *bind* and *connect*. _bind_(2) belongs to the *bind* group, whereas the
other system calls belong to the *connect* group. See the section called *Write
Sandboxing* for more information on how to customize this behaviour.

## Lock Sandboxing

This sandboxing utilizes the *Landlock Linux Security Module* for simple
unprivileged access control. This sandboxing type is not dynamic and is applied
at the kernel level on startup. The sandboxing may be turned on with the
*sandbox/lock:on* sandbox command, and _read-only_, and _read-write_ allowlists
can be populated using the sandbox commands *allow/lock/read+/path* and
*allow/lock/write+/path*. Note the syd process is also included in this
sandbox for added security such that a compromised syd process is still stuck
inside the *Landlock* sandbox.

## Memory Sandboxing

This sandboxing handles the system calls _brk_(2), _mmap_(2), _mmap2_(2),
and _mremap_(2) and checks the per-process memory usage on each memory
allocation request. If the memory usage reaches the maximum value
defined by *mem/max*, the system call is denied with *ENOMEM*. Moreover
the virtual memory size can be limited using *mem/vm_max*. If the limit
is reached on the entry of any of the respective system calls, the
system call is denied with *ENOMEM*. If *mem/kill* is set to
true, the signal *SIGKILL* is delivered to the offending process.

The per-process memory usage is a fair estimate calculated using the file
*/proc/pid/smaps* summing the following fields together:

. *"Pss" (Proportional Set Size)* is similar to **Rss**¹, but \
accounts for shared memory more accurately by dividing it \
among the processes that share it.
. *"Private_Dirty"* represents the private memory that has \
been modified (dirty).
. *"Shared_Dirty"* represents the shared memory that has \
been modified.

¹: *"Rss" (Resident Set Size)* is the portion of memory occupied by a
process that is held in RAM.

Note, _memory sandboxing is not an alternative to cgroups_. You should use
_cgroups_(7) when you can instead. This is meant for more constrained
environments where _cgroups_(7) is not supported or not available due to missing
permissions or other similar restrictions.

Warning, _this is not a strict limit_, syd is reliant on the allocation
granularity of the processes in the sandbox. Programs will be able to overstep
the limit at least by amount of memory they allocate at once.

## PID sandboxing

This sandboxing handles the system calls _fork_(2), _vfork_(2), _clone_(2), and
_clone3_(2) and checks the total number of tasks running on the system on each
process creation request. If the count reaches the maximum value defined by
*pid/max*, the system call is denied with *EACCES*. If *pid/kill* is set to
true, the signal *SIGKILL* is delivered to the offending process.

This sandboxing type is best coupled with a pid namespace using *unshare/pid*.
In this mode, syd will check the number of running tasks _in the current
namespace only._

Note, _PID sandboxing is not an alternative to cgroups_. You should use
_cgroups_(7) when you can instead. This is meant for more constrained
environments where _cgroups_(7) is not supported or not available due to missing
permissions or other similar restrictions.

# PROFILES

|[ *container*
:< Enables Linux namespaces.
|[ *immutable*
:< Enables Linux namespaces and remounts the following directories _read only_ in the new mount namespace:
|[
:< */etc*, */home*, */media*, */mnt*, */opt*, */srv*, and */usr*.
|[
:< Further mount options such as *nodev*, *noexec*, *nosuid*, and *noatime* are also applied as necessary.
|[
:< In addition, */dev/shm* and */tmp* are mounted private.
|[
:< See *syd-cat immutable* to see the full list of mount options.
|[ *landlock*
:< Enables *LandLock* and allows system directories for *Lock Sandboxing*.
|[ *paludis*
:< Used by the *Paludis* package mangler.
|[ *noipv4*
:< Disables *IPv4* connectivity.
|[ *noipv6*
:< Disables *IPv6* connectivity.
|[ *silent*
:< Silences all access violations.
|[ *lib*
:< *libsyd* helper profile. Turns all sandboxing off.
|[
:< Useful to configure syd in the application using *libsyd*.
|[ *user*
:< Allows user-specific directories, and connections, and parses the following files if they exist:
|[
:< */etc/user.syd-3*
|[
:< *~/.user.syd-3*

## Stacking Profiles

It is possible to stack multiple profiles to configure a more restricted
sandbox. Remember the order you stack the profiles matter, _the last matching
rule wins_.

## Login shell and the User Profile

When invoked without arguments, */bin/sh* is executed under syd with the
*user* profile as a login shell, use *SYD_SH* environment variable to override
the shell to execute.

# SECURITY

syd stands out for its ability to operate without requiring elevated
privileges, eliminating the need for root access or ptrace capabilities. This
feature significantly simplifies setup and usage. Users benefit from the
capability to dynamically configure the sandbox from within, with options to
secure it further as needed. However, the implementation exercises caution,
especially in mitigating *time-of-check to time-of-use (TOCTOU)*
vulnerabilities. syd employs *SECCOMP_USER_NOTIF_FLAG_CONTINUE* with
considerable restraint, primarily within the hooks for _exec_(2) and _chdir_(2)
system calls. These calls proceed directly in the sandbox process due to the
inability to emulate them fully within _seccomp_(2)'s constraints, this may
change in the future when the Linux kernel gains support to allow seccomp to
emulate them. For detailed information on specific limitations and their
security implications, refer to the *BUGS* section.

*Tip*: To take a quick peek at the seccomp filters applied by syd under
various different configurations, use *syd <flags...> -Epfc*. In addition, if
you export the environment variable *SYD_SECX* when invoking this command,
syd will also export the seccomp-bpf filters it applies to the syd process
itself.

## Accessing remote process memory

syd denies various system calls which can access remote process memory such
as *ptrace* and *process_vm_writev* and common sandboxing profiles such as
*paludis* and *user* disallow write access to the */proc/pid/mem* file. This
makes *TOCTOU* attack vectors harder to realize. See *syd-ls deny* to see
the full list of denied system calls.

## Changing UID/GID and groups

Since 3.1.9, the set-id family system calls, *setuid*, *setgid* et al., are
no-ops under syd. Since 3.8.6, the syd process applies a seccomp filter to
itself as well to get the same behaviour. This means even a compromised syd
process cannot elevate privileges using these system calls. Making these system
calls no-ops rather than denying them provides easy compatibility with daemons
that e.g. drop user:group to *nobody:nogroup*. See *syd-ls setid* to see
the full list of system calls in this group.

## Denying TIOCSTI _ioctl_(2)

The restriction on the use of the *TIOCSTI* _ioctl_(2) within the syd sandbox
addresses a significant security vulnerability associated with terminal input
injection. The *TIOCSTI* _ioctl_(2) allows a byte to be inserted into the
terminal input queue, effectively simulating keyboard input. This capability,
while potentially useful for legitimate purposes, poses
_a substantial security risk_, especially in scenarios where a process might
retain access to a terminal beyond its intended lifespan. Malicious use of this
_ioctl_(2) can lead to the injection of commands that execute with the
privileges of the terminal's owning process, thereby breaching the security
boundaries intended by user permissions and process isolation mechanisms.

The concern over *TIOCSTI* is well-documented in the security community. For
example, *OpenBSD* has taken measures to mitigate the risk by disabling the
*TIOCSTI* _ioctl_(2), reflecting its stance on the _ioctl_(2) as
_one of the most dangerous_ due to its potential for abuse in command injection
attacks. The decision to disable or restrict *TIOCSTI* in various Unix-like
operating systems underscores the _ioctl_(2)'s inherent security implications,
particularly in the context of privilege escalation and the execution of
unauthorized commands within a secured environment.

In summary, the restriction on *TIOCSTI* within syd is a critical security
measure that prevents a class of vulnerabilities centered around terminal input
injection, safeguarding against unauthorized command execution and privilege
escalation. This precaution aligns with broader security best practices and
mitigations adopted by the security community to address known risks associated
with terminal handling and process isolation.

## Restricting _ioctl_(2) request space and trace/allow_unsafe_ioctl

syd strategically narrows the range of permissible _ioctl_(2) requests within the
sandbox environment. This focused approach to _ioctl_(2) management is pivotal in
mitigating potential vulnerabilities associated with unrestricted _ioctl_(2) access.
By selectively enabling a subset of _ioctl_(2) requests, syd ensures that only
those necessary for the operational integrity and security requirements of
sandboxed processes are allowed. The list includes, but is not limited to:

*File Descriptor Manipulation*: Commands like *FIOCLEX* and *FIONCLEX* are
crucial for controlling the inheritance of file descriptors by executed
programs, thereby preventing accidental data leaks across processes.

*Non-blocking I/O*: *FIONBIO* and *FIOASYNC* facilitate the transition of file
descriptors between blocking and non-blocking modes, crucial for asynchronous
I/O operations.

*Terminal Handling*: A range of terminal control operations, such as *TCGETS*,
*TCSETS*, and window size adjustments with *TIOCGWINSZ* and *TIOCSWINSZ*, are
permitted to ensure the proper functioning of terminal-based interactions.

*Pseudoterminal Management*: Commands like *TIOCGPTN*, *TIOCGDEV*, and
*TIOCSPTLCK* allow for the secure management of pseudoterminals, essential for
the isolation of terminal sessions.

*Signal and Line Control*: *TIOCSIG* and *TIOCVHANGUP*, among others, provide
controlled mechanisms for signal delivery and line management within the
sandbox, contributing to the robustness of process and session handling.

In syd, running complex applications like _qemu_(1) or _firefox_(1), which
demand extensive _ioctl_(2) operations for *KVM* or *DRM*, necessitates the
relaxation of default _ioctl_(2) restrictions. To enable the required _ioctl_(2)
requests for these applications, users can employ the
*trace/allow_unsafe_ioctl:1* setting on startup. This adjustment, while
facilitating necessary functionality, introduces a potential increase in
security risks by expanding the set of permissible _ioctl_(2) requests. Further
refining the _ioctl_(2) request space for more sophisticated confinement is
planned for the future.

## Denying PR_SET_MM _prctl_(2)

The *PR_SET_MM* _prctl_(2) call allows processes with the *CAP_SYS_RESOURCE*
capability to adjust their memory map descriptors, facilitating operations like
self-modifying code by enabling dynamic changes to the process's memory layout.
For enhanced security, especially in constrained environments like syd, this
capability is restricted to prevent unauthorized memory manipulations that could
lead to vulnerabilities such as code injection or unauthorized code execution.
Notably, syd proactively drops *CAP_SYS_RESOURCE* among other capabilities at
startup to minimize security risks. This action is part of syd's broader
security strategy to limit potential attack vectors by restricting process
capabilities. Users can query *syd-ls drop* for a comprehensive list of
capabilities that syd relinquishes upon initialization, reinforcing the
sandbox's commitment to a secure execution environment by adhering to the
principle of least privilege.

## Restricting _prctl_(2) option space and trace/allow_unsafe_prctl

syd meticulously confines the scope of permissible _prctl_(2) operations to
enhance security within its sandbox environment. By limiting available
_prctl_(2) options to a specific set, including but not limited to
*PR_SET_PDEATHSIG*, *PR_GET_DUMPABLE*, *PR_SET_NO_NEW_PRIVS*, and
*PR_SET_SECCOMP*, syd ensures that only necessary process control
functionalities are accessible, thereby reducing the risk of exploitation
through less scrutinized _prctl_(2) calls. This constraint is pivotal in
preventing potential security vulnerabilities associated with broader _prctl_(2)
access, such as unauthorized privilege escalations or manipulations of process
execution states. However, recognizing the need for flexibility in certain
scenarios, syd offers the option to lift these restrictions through the
*trace/allow_unsafe_prctl:1* setting. This capability allows for a tailored
security posture, where users can opt for a more permissive _prctl_(2)
environment if required by their specific use case, while still maintaining
awareness of the increased security risks involved.

## Restricting _io_uring_(7) interface and trace/allow_unsafe_uring

The _io_uring_(7) interface can be used to _bypass path sandboxing_. By default,
syd restricts _io_uring_(7) operations due to their ability to perform system
calls that could undermine the sandbox's security controls, particularly those
designed to limit file access and modify file permissions.  The setting,
*trace/allow_unsafe_uring*, when enabled, relaxes these restrictions, allowing
_io_uring_(7) operations to proceed unimpeded. While this can significantly
enhance I/O performance for applications that rely on _io_uring_(7) for
efficient asynchronous operations, it requires careful consideration of the
security implications, ensuring that its use does not inadvertently compromise
the sandboxed application's security posture. Use *syd-ls uring* to see the full
list of system calls that belong to the io_uring interface.

## Restricting creation of device special files

Since version 3.1.12, syd has enhanced its security model by disallowing the
creation of device special files through the *mknod* and *mknodat* system calls.
This decision is rooted in mitigating potential security vulnerabilities, as
device special files could be exploited to circumvent established path-based
access controls within the sandbox environment. These files, which include
character and block devices, can provide direct access to hardware components or
facilitate interactions with kernel modules that could lead to unauthorized
actions or data exposure. By restricting their creation, syd significantly
reduces the risk of such exploit paths, reinforcing the integrity and security
of the sandboxed applications. This measure ensures that only predefined types
of files—such as FIFOs, regular files, and sockets—are permissible, aligning
with the principle of least privilege by limiting file system operations to
those deemed safe within the sandbox's context.

## Sharing Pid namespace with signal protections

Since version 3.6.7, syd has introduced a crucial security feature that
prevents sandboxed processes from sending signals to the syd process or any
of its threads. This protection is implemented by hooking and monitoring system
calls related to signal operations, including _kill_(2), _tkill_(2),
_tgkill_(2), and _pidfd_open_(2). When a sandboxed process attempts to send a
signal to syd or its threads, these system calls are intercepted, and the
operation is denied at the seccomp level with an *EACCES* error. This measure
ensures that syd maintains control over the execution and management of
sandboxed processes, safeguarding against interruptions or unauthorized
interactions that could compromise the security or stability of the sandbox
environment.

This security mechanism is part of syd's broader strategy to share the same
root, private proc, and mount namespaces with the sandboxed process,
facilitating secure and simple system call emulation. By making syd and its
threads immune to signals from sandboxed processes, the integrity and isolation
of the sandboxed environment are significantly enhanced, preventing potential
exploitation scenarios where sandboxed processes could disrupt the operation of
the sandbox manager or interfere with other sandboxed processes.

## System Call Handler Priority and Resource Management

Since version 3.8.1, syd has been implementing strategies to ensure the
smooth operation of the host system while managing security through its
sandboxing mechanism. It sets the _nice_(2) value of its system call handler
threads to *19*, ensuring these threads operate at _the lowest priority_ to minimize
CPU starvation for other critical processes. This approach prioritizes system
stability and fair CPU resource distribution, enabling syd to handle numerous
system calls without compromising the host's performance and responsiveness.

Enhancing this strategy, syd introduced further adjustments in versions 3.8.6
and 3.9.7 to address I/O and CPU resource management more comprehensively. From
version 3.8.6, it sets the I/O priority of the system call handler threads to
*idle*, ensuring that I/O operations do not monopolize resources and lead to I/O
starvation for other processes. Similarly, from version 3.9.7, it adjusts the
CPU scheduling priority of these threads to *idle*, further safeguarding against
CPU starvation. These measures collectively ensure that syd maintains optimal
performance and system responsiveness while securely sandboxing applications,
striking a balance between security enforcement and efficient system resource
utilization.

## Simplified Filesystem Access for Legacy Daemons

Since version 3.12.0, syd has adopted a nuanced approach to managing
filesystem access controls, specifically concerning the _chroot_(2) and
_pivot_root_(2) system calls. In syd, these calls are treated as
no-operations (no-ops), meaning they _report success without performing any
action_. This decision is strategically aimed at accommodating older daemons
that invoke *chroot(/var/empty)* as part of their initialization sequence,
typically after completing necessary path accesses. By rendering _chroot_(2) and
_pivot_root_(2) as no-ops at the seccomp-bpf kernel level, syd ensures
compatibility with such legacy software without introducing complexities in
syscall emulation.

The rationale behind this approach stems from the necessity of maintaining a
unified filesystem root between the sandbox and the sandboxed processes to
simplify syscall emulation. However, certain daemons, like OpenNTPD,
traditionally employ _chroot_(2) to a minimal directory (e.g., */var/empty*) as a
security measure. syd's no-op policy for these system calls allows it to host
these daemons without necessitating alterations to their source code or
operational logic, thus broadening the utility of syd in securing legacy
applications without sacrificing operational integrity or complicating the
sandbox's implementation.

## Streamlining File Synchronization Calls

Since version 3.8.8, syd has rendered the _sync_(2) and _syncfs_(2) system
calls as no-operations (no-ops), ensuring they report success without executing
any underlying functionality. This adjustment is designed to streamline
operations within the sandboxed environment, bypassing the need for these file
synchronization actions that could otherwise impact performance or complicate
the sandbox's control over file system interactions. By adopting this approach,
syd enhances its compatibility with applications that issue these calls,
without altering the sandboxed process's behavior or the integrity of file
system management.

## Restricting Resource Limits, Core Dumps, and trace/allow_unsafe_prlimit

Since version 3.9.6, syd has implemented restrictions on setting process
resource limits and generating core dumps for the sandboxed process, enhancing
the sandbox's security posture. This measure prevents the sandboxed process from
altering its own resource consumption boundaries or producing core dumps, which
could potentially leak sensitive information or be exploited for bypassing
sandbox restrictions. However, recognizing the need for flexibility in certain
use cases, syd provides the option to disable these restrictions at startup
through the *trace/allow_unsafe_prlimit:1* setting. This allows administrators
to tailor the sandbox's behavior to specific requirements, balancing security
considerations with functional needs.

## Enhancing Sandbox Security with Landlock

Since version 3.0.1, syd leverages the *Landlock Linux Security Module (LSM)*
to enforce advanced filesystem sandboxing, significantly bolstering the security
framework within which sandboxed processes operate. By integrating Landlock,
syd empowers even unprivileged processes to create secure sandboxes, enabling
fine-grained access control over filesystem operations without requiring
elevated permissions. This approach is instrumental in mitigating the risk of
security breaches stemming from bugs or malicious behaviors in applications,
offering a robust layer of protection by restricting ambient rights, such as
global filesystem or network access.

Landlock operates by allowing processes to self-impose restrictions on their
access to system resources, effectively creating a secure environment that
limits their operation to a specified set of files and directories. This
mechanism is particularly useful for running legacy daemons or applications that
require specific environmental setups, as it allows for the precise tailoring of
access rights, ensuring processes can only interact with designated parts of the
filesystem. For instance, by setting Landlock rules, syd can confine a
process's filesystem interactions to read-only or read-write operations on
explicitly allowed paths, thus preventing unauthorized access to sensitive areas
of the system.

Furthermore, the inclusion of the syd process itself within the
Landlock-enforced sandbox adds an additional layer of security. This design
choice ensures that even if the syd process were compromised, the attacker's
ability to manipulate the sandboxed environment or access unauthorized resources
would be significantly constrained. This self-sandboxing feature underscores
syd's commitment to maintaining a high security standard, offering peace of
mind to users by ensuring comprehensive containment of sandboxed processes.

## Namespace Isolation in syd

syd enhances sandbox isolation through meticulous namespace use, starting
from version 3.0.2. Version 3.9.10 marks a pivotal enhancement by restricting
user subnamespace creation, addressing a key path sandboxing bypass
vulnerability.  This strategic limitation thwarts sandboxed processes from
altering their namespace environment to access restricted filesystem areas.
Furthermore, since version 3.11.2, syd maintains process capabilities within
user namespaces, mirroring the _unshare_(1) command's *--keep-caps* behavior.
This ensures sandboxed processes retain necessary operational capabilities,
enhancing security without compromising functionality. Additionally, syd
utilizes the powerful *bind* command within the mount namespace to create
secure, isolated environments by allowing specific filesystem locations to be
remounted with custom attributes, such as *read-only* or *no-exec*, providing a
flexible tool for further restricting sandboxed processes' access to the
filesystem.

syd also introduces enhanced isolation within the mount namespace by
offering options to bind mount temporary directories over */dev/shm* and
*/tmp*, ensuring that sandboxed processes have private instances of these
directories. This prevents inter-process communication through shared memory
and mitigates the risk of temporary file-based attacks, further solidifying the
sandbox's defense mechanisms. Moreover, since version 3.11.2, syd mounts
the proc filesystem privately with the *hidepid=2* option, enhancing privacy by
concealing process information from unauthorized users.

syd's *container* and *immutable* profiles exemplify its adaptability,
offering from isolated to highly restrictive environments. The *container*
profile provides a general-purpose sandbox, while the *immutable* profile
enforces stricter controls, such as making essential system directories
read-only, to prevent tampering. This comprehensive approach underlines syd's
adept use of kernel features for robust sandbox security, ensuring a secure and
controlled execution environment for sandboxed applications. See *syd-cat
container*, and *syd-cat immutable* to list the rules in these sandboxing
profiles.

## Restricting environment and trace/allow_unsafe_environment

syd, from version 3.11.1, has implemented measures to clear unsafe
environment variables, such as *LD_PRELOAD*, enhancing security by preventing
the manipulation of dynamic linker behavior by sandboxed processes. This action
mitigates risks associated with dynamic linker hijacking, where adversaries may
load malicious shared libraries to execute unauthorized code, potentially
leading to privilege escalation, persistence, or defense evasion. Variables like
*LD_PRELOAD* allow specifying additional shared objects to be loaded before any
others, which could be exploited to override legitimate functions with malicious
ones, thus hijacking the execution flow of a program. To accommodate scenarios
where developers might need to use these variables for legitimate purposes,
syd allows this security feature to be disabled at startup with
*trace/allow_unsafe_environment:1*, offering flexibility while maintaining a
strong security posture. This careful balance ensures that sandboxed
applications operate within a tightly controlled environment, significantly
reducing the attack surface and enhancing the overall security framework within
which these applications run.

## Managing Linux Capabilities for Enhanced Security

Since its 3.0.17 release, syd strategically curtails specific Linux
_capabilities_(7) for sandboxed processes to bolster security. By revoking privileges
such as *CAP_SYS_ADMIN* among others, syd significantly reduces the risk of
privilege escalation and system compromise. This proactive measure ensures that
even if a sandboxed process is compromised, its ability to perform sensitive
operations is severely limited. The comprehensive list of dropped capabilities,
including but not limited to *CAP_DAC_OVERRIDE*, *CAP_NET_ADMIN*, and
*CAP_SYS_RAWIO*, reflects a meticulous approach to minimizing the attack surface.
See *syd-ls drop* to see the full list of _capabilities_(7) that syd drops at
startup.

Exceptions to this stringent policy, introduced in version 3.11.1, such as
maintaining *CAP_NET_RAW* with *trace/allow_unsafe_socket_families:1* and
*CAP_SYS_TIME* with *trace/allow_unsafe_adjtime:1*, offer a nuanced security
model. These exceptions allow for necessary network and time adjustments within
the sandbox, providing flexibility without significantly compromising security.

This balanced strategy of restricting _capabilities_(7), coupled with selective
permissions, exemplifies syd's commitment to crafting a secure yet functional
sandbox environment. By leveraging the granularity of Linux _capabilities_(7),
syd offers a robust framework for safeguarding applications against a variety
of threats, underscoring its role as a pivotal tool in the security arsenal of
Linux environments.

# HISTORY & DESIGN

- *sydbox-0* <https://git.sr.ht/~alip/syd/tree/sydbox-0> is a _ptrace_(2) based sandbox.
- *sydbox-1* <https://git.sr.ht/~alip/syd/tree/sydbox-1> is a _ptrace_(2) and _seccomp_(2) based sandbox.
- *sydbox-2* <https://git.sr.ht/~alip/syd/tree/sydbox-1> is a _seccomp_(2) and _seccomp-notify_ based sandbox.
- *sydbox-3* is a rewrite of *sydbox-2* in Rust and it's what you are looking at.

This codebase has a history of a bit over 15 years and up to this point we have
used *C11* as our implementation language for various reasons. With *sydbox-3*
we are moving forwards one step and writing the sandbox from scratch using the
*Rust* programming language with the only non-Rust dependency being *libseccomp*.
Although we inherit many ideas and design decisions from the old codebase, we
also don't shy away from radically changing the internal implementation making
it much simpler, idiomatic, and less prone to bugs. We have _proper multiarch
support_ since release 3.0.11, e.g on x86-64, you can run your x32 or x86
binaries just fine under Syd.

This version takes advantage of multithreading and handles system calls using a
thread pool whose size is equal to the number of CPUs on the running machine and
utilizes globsets to match a list of patterns at once, thus continues to perform
reasonably well even with very long rulesets. Moreover, access violations are
logged into _syslog_(3), so you may use a command like *journalctl
SYSLOG_IDENTIFIER=syd* (or shortly *syd-log*) to view the sandbox logs. This
version also comes with four new sandboxing types called *Lock Sandboxing*,
*Memory Sandboxing*, *PID sandboxing*, *Stat Sandboxing*: *Lock Sandboxing*
utilizes the Landlock Linux Security Module (LSM), *Memory Sandboxing* allows
the user to define a per-process memory limit, *PID sandboxing* allows the user
to define a limit on the maximum number of running tasks under the sandbox,
whereas *Stat Sandboxing* can be used to effectively _hide files and
directories_ from the sandboxed process.

Finally, the new syd has support for namespaces. Use e.g. *syd -m
unshare/user:1* to create a user namespace. You may use *mount*, *uts*, *ipc*,
*pid*, *net*, and *cgroup* instead of *user* to create various namespaces. You
may use the *container* profile as a shorthand to create namespaces with
*syd -pcontainer*.

You may use syd as your login shell because it is very practical to have a
restricted user. To do this simply add */path/to/syd* to the file */etc/shells*
and do *chsh -s /path/to/syd username* as root. In this mode the sandbox may be
configured using the files */etc/user.syd-3* and *~/.user.syd-3*. If you want to
restrict user configuration of the sandbox, lock the sandbox using *lock:on* at
the end of the site-wide configuration file.

# EXHERBO

syd is the default sandbox of *Exherbo Linux*. We use it to provide a restricted
environment under which package builds run with controlled access to file system
and network resources. *exheres-0* has a function called *esandbox* to interact
with syd.

# BUGS

In the operation of syd, certain system calls are not fully emulated due to
_seccomp_(2) limitations, resulting in the sandbox process continuing these
calls directly. These include _execve_(2), _execveat_(2) for execution,
_chdir_(2), _fchdir_(2) for directory changes, and _open_(2) operations with
*O_PATH* flag. Consequently, this behavior exposes vulnerabilities to
*time-of-check to time-of-use* attacks, allowing for the circumvention of *Exec
Sandboxing* to execute denylisted paths, the bypass of *Stat Sandboxing* for
unauthorized directory access _without disclosing directory contents_ (owing to
_getdents_(2) call emulation), and the detection of hidden files _without
revealing file metadata_, as _stat_(2) calls are emulated.

# SEE ALSO

_syd_(1), _syd_(2), _syd_(5), _seccomp_(2), _pidfd_getfd_(2), _pidfd_send
signal_(2), _ioctl_(2), _ioctl_tty_(2), _prctl_(2), _namespaces_(7),
_cgroup_namespaces_(7), _ipc_namespaces_(7), _mount_namespaces_(7),
_network_namespaces_(7), _pid_namespaces_(7), _user_namespaces_(7),
_uts_namespaces_(7)

- https://exherbolinux.org/docs/eapi/exheres-for-smarties.html#sandboxing

# AUTHORS

Maintained by Ali Polatel. Up-to-date sources can be found at
https://git.sr.ht/~alip/syd and bugs/patches can be submitted by email to
~alip/sydbox-devel@lists.sr.ht.
